package com.ningdatech.pmapi.gov.convert;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.ningdatech.pmapi.gov.contants.BizProjectContant;
import com.ningdatech.pmapi.gov.model.dto.*;
import com.ningdatech.pmapi.gov.model.req.*;
import com.ningdatech.pmapi.gov.model.vo.*;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname ProjectConvert
 * @Description
 * @Date 2023/8/25 11:53
 * @Author PoffyZhang
 */
public class ProjectConvert {

    /**
     * 6个 转换方法
     * @param saveDTO
     * @return
     */
    public static ProjectPushReq convert(GovBizProjectSaveDTO saveDTO) {
        ProjectPushReq req = new ProjectPushReq();
        req.setBaseinfo(convertBase(saveDTO.getBaseinfo(),BizProjectContant.EFFECTIVE_CODE));
        req.setApply(convertApply(saveDTO.getApply()));
        req.setApprove(convertApprove(saveDTO.getApprove()));
        req.setCimplement(convertCimplement(saveDTO.getCimplement()));
        req.setMimplement(convertMimplement(saveDTO.getMimplement()));
        req.setProcures(convertProcures(saveDTO.getProcures()));
        return req;
    }

    public static ProjectPushReq convert(GovBizProjectDetailVO vo) {
        ProjectPushReq req = new ProjectPushReq();
        req.setBaseinfo(convertBaseVo(vo.getBaseinfo(),BizProjectContant.EFFECTIVE_CODE));
        req.setApply(convertApplyVo(vo.getApply()));
        req.setApprove(convertApproveVo(vo.getApprove()));
        req.setCimplement(convertCimplementVo(vo.getCimplement()));
        req.setMimplement(convertMimplementVo(vo.getMimplement()));
        req.setProcures(convertProcuresVo(vo.getProcures()));
        return req;
    }

    private static List<ProjectProcureReq> convertProcures(List<GovBizProjectProcureDTO> procures) {
        if(CollUtil.isEmpty(procures)){
            return Collections.emptyList();
        }

        List<ProjectProcureReq> req = procures.stream()
                .map(p -> {
                    ProjectProcureReq r = BeanUtil.copyProperties(p, ProjectProcureReq.class);
                    if(Objects.nonNull(p.getId())){
                        r.setTongID(p.getId().intValue());
                    }
                    return r;
                })
                .collect(Collectors.toList());
        return req;
    }

    private static List<ProjectProcureReq> convertProcuresVo(List<GovBizProjectProcureVO> procures) {
        if(CollUtil.isEmpty(procures)){
            return Collections.emptyList();
        }

        List<ProjectProcureReq> req = procures.stream()
                .map(p -> {
                    ProjectProcureReq r = BeanUtil.copyProperties(p, ProjectProcureReq.class);
                    if(Objects.nonNull(p.getId())){
                        r.setTongID(p.getId().intValue());
                    }
                    return r;
                })
                .collect(Collectors.toList());
        return req;
    }

    private static ProjectMimplementReq convertMimplement(GovBizProjectMimplementDTO mimplement) {
        ProjectMimplementReq req = BeanUtil.copyProperties(mimplement,ProjectMimplementReq.class);
        return req;
    }

    private static ProjectMimplementReq convertMimplementVo(GovBizProjectMimplementVO mimplement) {
        ProjectMimplementReq req = BeanUtil.copyProperties(mimplement,ProjectMimplementReq.class);
        return req;
    }

    public static ProjectCimplementReq convertCimplement(GovBizProjectCimplementDTO cimplement) {
        ProjectCimplementReq req = BeanUtil.copyProperties(cimplement,ProjectCimplementReq.class);
        return req;
    }

    public static ProjectCimplementReq convertCimplementVo(GovBizProjectCimplementVO cimplement) {
        ProjectCimplementReq req = BeanUtil.copyProperties(cimplement,ProjectCimplementReq.class);
        return req;
    }

    public static ProjectApproveReq convertApprove(GovBizProjectApproveDTO approve) {
        ProjectApproveReq req = new ProjectApproveReq();
        if(Objects.isNull(approve)){
            return req;
        }
        req.setBaseProjId(approve.getBaseProjId());
        req.setBaseProjName(approve.getBaseProjName());
        req.setOp(approve.getOp());
        req.setApprovalFile(approve.getApprovalFile());
        req.setBaseReviewCommentsFile(approve.getBaseReviewCommentsFile());
        req.setPreliminaryDesignFile(approve.getPreliminaryDesignFile());
        req.setAreaCode(approve.getAreaCode());
        req.setBaseDevelopCode(approve.getBaseDevelopCode());
        req.setBaseExpertTotalMoney(String.valueOf(approve.getBaseExpertTotalMoney()));
        req.setBaseExpertYearMoney(String.valueOf(approve.getBaseExpertYearMoney()));
        req.setReleaseYearMoney(String.valueOf(approve.getReleaseYearMoney()));
        req.setBaseInitialReviewTotalMoney(String.valueOf(approve.getBaseInitialReviewTotalMoney()));
        req.setBaseReviewOpinion(approve.getBaseReviewOpinion());
        req.setBaseReviewResults(approve.getBaseReviewResults());
        req.setEqualProtectionLevel(approve.getEqualProtectionLevel());
        req.setPreliminaryDesignScheme(approve.getPreliminaryDesignScheme());
        req.setSetProjCodeFinan(approve.getSetProjCodeFinan());
        return req;
    }

    public static ProjectApproveReq convertApproveVo(GovBizProjectApproveVO approve) {
        ProjectApproveReq req = new ProjectApproveReq();
        if(Objects.isNull(approve)){
            return req;
        }
        req.setBaseProjId(approve.getBaseProjId());
        req.setBaseProjName(approve.getBaseProjName());
        req.setOp(approve.getOp());
        req.setApprovalFile(approve.getApprovalFile());
        req.setBaseReviewCommentsFile(approve.getBaseReviewCommentsFile());
        req.setPreliminaryDesignFile(approve.getPreliminaryDesignFile());
        req.setAreaCode(approve.getAreaCode());
        req.setBaseDevelopCode(approve.getBaseDevelopCode());
        req.setBaseExpertTotalMoney(String.valueOf(approve.getBaseExpertTotalMoney()));
        req.setBaseExpertYearMoney(String.valueOf(approve.getBaseExpertYearMoney()));
        req.setReleaseYearMoney(String.valueOf(approve.getReleaseYearMoney()));
        req.setBaseInitialReviewTotalMoney(String.valueOf(approve.getBaseInitialReviewTotalMoney()));
        req.setBaseReviewOpinion(approve.getBaseReviewOpinion());
        req.setBaseReviewResults(approve.getBaseReviewResults());
        req.setEqualProtectionLevel(approve.getEqualProtectionLevel());
        req.setPreliminaryDesignScheme(approve.getPreliminaryDesignScheme());
        req.setSetProjCodeFinan(approve.getSetProjCodeFinan());
        return req;
    }

    public static ProjectApplyReq convertApply(GovBizProjectApplyDTO apply) {
        ProjectApplyReq req = new ProjectApplyReq();
        if(Objects.isNull(apply)){
            return req;
        }
        req.setOp(apply.getOp());
        req.setBaseProjId(apply.getBaseProjId());
        req.setBaseProjName(apply.getBaseProjName());
        req.setAreaCode(apply.getAreaCode());
        req.setBaseAccountAppName(apply.getBaseAccountAppName());
        req.setBaseBasisEstablish(apply.getBaseBasisEstablish());
        req.setBaseCoreBusiness(apply.getBaseCoreBusiness());
        req.setBaseCoreBusinessCode(apply.getBaseCoreBusinessCode());
        req.setBaseBrainName(apply.getBaseBrainName());
        req.setBaseDevelopCode(apply.getBaseDevelopCode());
        req.setBaseLowestLevel(apply.getBaseLowestLevel());
        req.setBaseHistorProjId(apply.getBaseHistorProjId());
        req.setBaseHistorProjName(apply.getBaseHistorProjName());
        req.setBaseOperatMaintenFile(apply.getBaseOperatMaintenFile());
        req.setBaseProjApplyFile(apply.getBaseProjApplyFile());
        req.setBaseProjBasisFile(apply.getBaseProjBasisFile());
        req.setBaseProjOtherFile(apply.getBaseProjOtherFile());
        req.setBaseResearchReportFile(apply.getBaseResearchReportFile());
        req.setBaseProjConsClass(apply.getBaseProjConsClass());
        req.setBaseProjDuration(String.valueOf(apply.getBaseProjDuration()));
        req.setBaseProjIntro(apply.getBaseProjIntro());
        req.setBaseProjStartTime(String.valueOf(apply.getBaseProjStartTime()));
        req.setBaseProjEndTime(String.valueOf(apply.getBaseProjEndTime()));
        req.setBaseProjSetYear(apply.getBaseProjSetYear());
        req.setBaseProjSys(apply.getBaseProjSys());
        req.setBaseProjSysCode(apply.getBaseDevelopCode());
        req.setBaseProjDeclAmount(String.valueOf(apply.getBaseProjDeclAmount()));
        req.setBaseProjTotalAmount(String.valueOf(apply.getBaseProjTotalAmount()));
        req.setBaseBasisAmountOri(apply.getBaseBasisAmountOri());
        req.setBaseProjAmountOri(apply.getBaseProjAmountOri());
        req.setBaseProjRemark(apply.getBaseProjRemark());
        req.setBeseExpectedResults(apply.getBeseExpectedResults());
        return req;
    }

    public static ProjectApplyReq convertApplyVo(GovBizProjectApplyVO apply) {
        ProjectApplyReq req = new ProjectApplyReq();
        if(Objects.isNull(apply)){
            return req;
        }
        req.setOp(apply.getOp());
        req.setBaseProjId(apply.getBaseProjId());
        req.setBaseProjName(apply.getBaseProjName());
        req.setAreaCode(apply.getAreaCode());
        req.setBaseAccountAppName(apply.getBaseAccountAppName());
        req.setBaseBasisEstablish(apply.getBaseBasisEstablish());
        req.setBaseCoreBusiness(apply.getBaseCoreBusiness());
        req.setBaseCoreBusinessCode(apply.getBaseCoreBusinessCode());
        req.setBaseBrainName(apply.getBaseBrainName());
        req.setBaseDevelopCode(apply.getBaseDevelopCode());
        req.setBaseLowestLevel(apply.getBaseLowestLevel());
        req.setBaseHistorProjId(apply.getBaseHistorProjId());
        req.setBaseHistorProjName(apply.getBaseHistorProjName());
        req.setBaseOperatMaintenFile(apply.getBaseOperatMaintenFile());
        req.setBaseProjApplyFile(apply.getBaseProjApplyFile());
        req.setBaseProjBasisFile(apply.getBaseProjBasisFile());
        req.setBaseProjOtherFile(apply.getBaseProjOtherFile());
        req.setBaseResearchReportFile(apply.getBaseResearchReportFile());
        req.setBaseProjConsClass(apply.getBaseProjConsClass());
        req.setBaseProjDuration(String.valueOf(apply.getBaseProjDuration()));
        req.setBaseProjIntro(apply.getBaseProjIntro());
        req.setBaseProjStartTime(String.valueOf(apply.getBaseProjStartTime()));
        req.setBaseProjEndTime(String.valueOf(apply.getBaseProjEndTime()));
        req.setBaseProjSetYear(apply.getBaseProjSetYear());
        req.setBaseProjSys(apply.getBaseProjSys());
        req.setBaseProjSysCode(apply.getBaseDevelopCode());
        req.setBaseProjDeclAmount(String.valueOf(apply.getBaseProjDeclAmount()));
        req.setBaseProjTotalAmount(String.valueOf(apply.getBaseProjTotalAmount()));
        req.setBaseBasisAmountOri(apply.getBaseBasisAmountOri());
        req.setBaseProjAmountOri(apply.getBaseProjAmountOri());
        req.setBaseProjRemark(apply.getBaseProjRemark());
        req.setBeseExpectedResults(apply.getBeseExpectedResults());
        return req;
    }

    public static ProjectBaseInfoReq convertBase(GovBizProjectBaseinfoDTO baseinfo,String isEffective) {
        ProjectBaseInfoReq req = BeanUtil.copyProperties(baseinfo,ProjectBaseInfoReq.class);
        req.setIsEffective(isEffective);
        return req;
    }

    public static ProjectBaseInfoReq convertBaseVo(GovBizProjectBaseinfoVO baseinfo,String isEffective) {
        ProjectBaseInfoReq req = BeanUtil.copyProperties(baseinfo,ProjectBaseInfoReq.class);
        req.setIsEffective(isEffective);
        return req;
    }
}
