package com.ningdatech.pmapi.gov.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.gov.contants.BizProjectContant;
import com.ningdatech.pmapi.gov.convert.ProjectConvert;
import com.ningdatech.pmapi.gov.model.dto.*;
import com.ningdatech.pmapi.gov.model.entity.*;
import com.ningdatech.pmapi.gov.model.req.ProjectBaseInfoReq;
import com.ningdatech.pmapi.gov.model.req.ProjectPushReq;
import com.ningdatech.pmapi.gov.model.vo.*;
import com.ningdatech.pmapi.gov.service.*;
import com.ningdatech.pmapi.projectdeclared.utils.GenerateProjectCodeUtil;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.provincial.manage.ProvincialManage;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.user.entity.enumeration.RoleEnum;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.ningdatech.pmapi.wps.manage.WpsConvertManage;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zpf
 * @date 2023/8/21 下午2:27
 */

@Component
@RequiredArgsConstructor
@Slf4j
public class GovProjectCollectionManage {

    private final IGovProjectDictionaryService dictionaryService;

    private final IGovBizProjectApplyService projectApplyService;

    private final IGovBizProjectBaseinfoService baseinfoService;

    private final IGovBizProjectApproveService approveService;

    private final IGovBizProjectCimplementService cimplementService;

    private final IGovBizProjectMimplementService mimplementService;

    private final IGovBizProjectProcureService procureService;

    private final GenerateProjectCodeUtil generateProjectCodeUtil;

    private final IGovBizProjectDraftService draftService;

    private final RestTemplate restTemplate;

    private final WpsConvertManage wpsConvertManage;

    @Value("${project.push-url}")
    private String pushUrl;

    @Value("${project.no-effective-url}")
    private String noEffectiveUrl;

    @Value("${project.delete-all-url}")
    private String deleteAllUrl;

    public List<GovProjectDictionaryVO> dictionary(String type) {

        List<GovProjectDictionary> dictionaries = dictionaryService.list(Wrappers.lambdaQuery(GovProjectDictionary.class)
                .eq(StringUtils.isNotBlank(type), GovProjectDictionary::getType, type)
                .eq(GovProjectDictionary::getDeleted,Boolean.FALSE));

        if(CollUtil.isEmpty(dictionaries)){
            return Collections.emptyList();
        }

        return dictionaries.stream().map(d -> BeanUtil.copyProperties(d,GovProjectDictionaryVO.class))
                .collect(Collectors.toList());
    }

    /**
     * 分页查 项目归集
     * @param req
     * @return
     */
    public PageVo<GovBizProjectListVO> list(ProjectListReq req) {
        Page<GovBizProjectBaseinfo> page = req.page();
        LambdaQueryWrapper<GovBizProjectBaseinfo> wrapper = Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(StringUtils.isNotBlank(req.getProjectName()),GovBizProjectBaseinfo::getBaseProjName,req.getProjectName())
                .like(StringUtils.isNotBlank(req.getBuildOrg()),GovBizProjectBaseinfo::getBaseBuildDeprt,req.getBuildOrg())
                .eq(StringUtils.isNotBlank(req.getAreaCode()),GovBizProjectBaseinfo::getBaseAreaCode,req.getAreaCode() + BizConst.NINE_AREA_CODE_LAST)
                .eq(StringUtils.isNotBlank(req.getRegionCode()),GovBizProjectBaseinfo::getBaseAreaCode,req.getRegionCode() + BizConst.NINE_AREA_CODE_LAST)
                .gt(Objects.nonNull(req.getCreateOnMin()),GovBizProjectBaseinfo::getTongTime,req.getCreateOnMin())
                .le(Objects.nonNull(req.getCreateOnMax()),GovBizProjectBaseinfo::getTongTime,req.getCreateOnMax())
                .eq(GovBizProjectBaseinfo::getDeleted,Boolean.FALSE)
                .orderByDesc(GovBizProjectBaseinfo::getBizTime);
//        permissions(wrapper);
        baseinfoService.page(page,wrapper);

        if(CollUtil.isEmpty(page.getRecords())){
            return PageVo.empty();
        }

        List<String> projIds = page.getRecords().stream()
                .filter(p -> Objects.nonNull(p) && StringUtils.isNotBlank(p.getBaseProjId()))
                .map(GovBizProjectBaseinfo::getBaseProjId).collect(Collectors.toList());

        Map<String,GovBizProjectApply> applyMap = Maps.newHashMap();
        if(CollUtil.isNotEmpty(projIds)){
            List<GovBizProjectApply> applys = projectApplyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .in(GovBizProjectApply::getBaseProjId,projIds));
            applyMap = applys.stream().collect(Collectors.toMap(GovBizProjectApply::getBaseProjId,a -> a));
        }

        Map<String, GovBizProjectApply> finalApplyMap = applyMap;
        List<GovBizProjectListVO> res = page.getRecords().stream().map(p -> {
            GovBizProjectListVO vo = new GovBizProjectListVO();
            vo.setBaseAreaCode(p.getBaseAreaCode());
            vo.setBaseAreaName(p.getBaseAreaName());
            vo.setBaseBuildDeprt(p.getBaseBuildDeprt());
            vo.setBaseBuildDeprtDing(p.getBaseBuildDeprtDing());
            vo.setBaseProjId(p.getBaseProjId());
            vo.setBaseProjName(p.getBaseProjName());
            if(finalApplyMap.containsKey(p.getBaseProjId())){
                GovBizProjectApply apply = finalApplyMap.get(p.getBaseProjId());
                vo.setBaseProjDeclAmount(apply.getBaseProjDeclAmount());
                vo.setBaseProjTotalAmount(apply.getBaseProjTotalAmount());
                vo.setBaseProjSetYear(apply.getBaseProjSetYear());
            }
            return vo;
        }).collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 草稿箱列表
     * @param req
     * @return
     */
    public PageVo<GovBizProjectListVO> draftList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Page<GovBizProjectDraft> page = req.page();
        LambdaQueryWrapper<GovBizProjectDraft> wrapper = Wrappers.lambdaQuery(GovBizProjectDraft.class)
                .like(StringUtils.isNotBlank(req.getProjectName()),GovBizProjectDraft::getBaseProjName,req.getProjectName())
                .like(StringUtils.isNotBlank(req.getBuildOrg()),GovBizProjectDraft::getBaseBuildDeprt,req.getBuildOrg())
                .eq(StringUtils.isNotBlank(req.getAreaCode()),GovBizProjectDraft::getBaseAreaCode,req.getAreaCode() + BizConst.NINE_AREA_CODE_LAST)
                .eq(StringUtils.isNotBlank(req.getRegionCode()),GovBizProjectDraft::getBaseAreaCode,req.getRegionCode() + BizConst.NINE_AREA_CODE_LAST)
                .gt(Objects.nonNull(req.getCreateOnMin()),GovBizProjectDraft::getTongTime,req.getCreateOnMin())
                .le(Objects.nonNull(req.getCreateOnMax()),GovBizProjectDraft::getTongTime,req.getCreateOnMax())
                .eq(GovBizProjectDraft::getDeleted,Boolean.FALSE)
                .eq(GovBizProjectDraft::getCreateId,user.getUserId())
                .orderByDesc(GovBizProjectDraft::getBizTime);
        draftService.page(page,wrapper);

        if(CollUtil.isEmpty(page.getRecords())){
            return PageVo.empty();
        }

        List<GovBizProjectListVO> res = page.getRecords().stream().map(p -> {
            GovBizProjectListVO vo = new GovBizProjectListVO();
            vo.setBaseAreaCode(p.getBaseAreaCode());
            vo.setBaseAreaName(p.getBaseAreaName());
            vo.setBaseBuildDeprt(p.getBaseBuildDeprt());
            vo.setBaseBuildDeprtDing(p.getBaseBuildDeprtDing());
            vo.setBaseProjId(p.getBaseProjId());
            vo.setBaseProjName(p.getBaseProjName());
            vo.setBaseProjDeclAmount(p.getBaseProjDeclAmount());
            vo.setBaseProjTotalAmount(p.getBaseProjTotalAmount());
            vo.setBaseProjSetYear(p.getBaseProjSetYear());
            vo.setDraftId(p.getId());
            return vo;
        }).collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 数据权限
     * @param wrapper
     */
    private void permissions(LambdaQueryWrapper<GovBizProjectBaseinfo> wrapper) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        List<Role> userRoleList = user.getUserRoleList();
        for(Role role : userRoleList){
            //超管和区域管理员
            if(RoleEnum.SUPER_ADMIN.name().equals(role.getCode()) ||
                    RoleEnum.REGION_MANAGER.name().equals(role.getCode())){
                log.info("超管和区管 可以看所有");
            }else{
                //其他情况 都只能看 自己单位的
                wrapper.eq(GovBizProjectBaseinfo::getBaseBuildDeprtDing,user.getEmpPosUnitCode());
            }
        }
    }

    public Long count(ProjectListReq req) {
        LambdaQueryWrapper<GovBizProjectBaseinfo> wrapper = Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .like(StringUtils.isNotBlank(req.getProjectName()),GovBizProjectBaseinfo::getBaseProjName,req.getProjectName())
                .like(StringUtils.isNotBlank(req.getBuildOrg()),GovBizProjectBaseinfo::getBaseBuildDeprt,req.getBuildOrg())
                .eq(StringUtils.isNotBlank(req.getAreaCode()),GovBizProjectBaseinfo::getAreaCode,req.getAreaCode())
                .eq(StringUtils.isNotBlank(req.getRegionCode()),GovBizProjectBaseinfo::getAreaCode,req.getRegionCode())
                .gt(Objects.nonNull(req.getCreateOnMin()),GovBizProjectBaseinfo::getTongTime,req.getCreateOnMin())
                .le(Objects.nonNull(req.getCreateOnMax()),GovBizProjectBaseinfo::getTongTime,req.getCreateOnMax())
                .orderByDesc(GovBizProjectBaseinfo::getBizTime);
        return baseinfoService.count(wrapper);
    }

    /**
     * 项目归集详情
     * @param projId
     * @return
     */
    public GovBizProjectDetailVO detail(String projId) {
        GovBizProjectDetailVO res = new GovBizProjectDetailVO();
        //1.先查 基本信息
        GovBizProjectBaseinfo baseinfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, projId)
                .orderByDesc(GovBizProjectBaseinfo::getBizTime)
                .last(BizConst.LIMIT_1));

        if(Objects.isNull(baseinfo)){
            return res;
        }
        res.setBaseProjId(baseinfo.getBaseProjId());
        res.setBaseProjName(baseinfo.getBaseProjName());
        res.setBaseinfo(BeanUtil.copyProperties(baseinfo, GovBizProjectBaseinfoVO.class));

        //2.再查 申报信息
        GovBizProjectApply apply = projectApplyService.getOne(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .eq(GovBizProjectApply::getBaseProjId, projId)
                .orderByDesc(GovBizProjectApply::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(apply)){
            res.setApply(BeanUtil.copyProperties(apply, GovBizProjectApplyVO.class));
        }

        //3.再查 审批信息
        GovBizProjectApprove approve = approveService.getOne(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                .eq(GovBizProjectApprove::getBaseProjId, projId)
                .orderByDesc(GovBizProjectApprove::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(approve)){
            res.setApprove(BeanUtil.copyProperties(approve, GovBizProjectApproveVO.class));
        }

        //4 对应查询 实施项目还是运维项目
        GovBizProjectCimplement cimplement = cimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                .eq(GovBizProjectCimplement::getBaseProjId, projId)
                .orderByDesc(GovBizProjectCimplement::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(cimplement)){
            res.setCimplement(BeanUtil.copyProperties(cimplement, GovBizProjectCimplementVO.class));
        }
        GovBizProjectMimplement mimplement = mimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                .eq(GovBizProjectMimplement::getBaseProjId, projId)
                .orderByDesc(GovBizProjectMimplement::getBizTime)
                .last(BizConst.LIMIT_1));
        if(Objects.nonNull(mimplement)){
            res.setMimplement(BeanUtil.copyProperties(mimplement, GovBizProjectMimplementVO.class));
        }

        //5.最后查询采购信息
        List<GovBizProjectProcure> procures = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                .eq(GovBizProjectProcure::getBaseProjId, projId)
                .orderByDesc(GovBizProjectProcure::getBizTime));
        if(CollUtil.isNotEmpty(procures)){
            res.setProcures(procures.stream()
                    .map(p -> BeanUtil.copyProperties(p, GovBizProjectProcureVO.class))
                    .collect(Collectors.toList()));
        }

        return res;
    }

    /**
     * 草稿箱详情
     * @param draftId
     * @return
     */
    public GovBizProjectDetailVO draftDetail(Long draftId) {
        GovBizProjectDetailVO res = new GovBizProjectDetailVO();
        //查询草稿表
        GovBizProjectDraft draft = draftService.getById(draftId);

        if(Objects.isNull(draft)){
            return null;
        }
        res.setDraftId(draftId);

        //1.基本信息
        res.setBaseProjId(draft.getBaseProjId());
        res.setBaseProjName(draft.getBaseProjName());
        res.setBaseinfo(BeanUtil.copyProperties(draft, GovBizProjectBaseinfoVO.class));

        //2.申报信息
        res.setApply(BeanUtil.copyProperties(draft, GovBizProjectApplyVO.class));

        //3.再查 审批信息
        res.setApprove(BeanUtil.copyProperties(draft, GovBizProjectApproveVO.class));

        //4 对应查询 实施项目还是运维项目
        GovBizProjectCimplementVO cimplementVo = BeanUtil.copyProperties(draft, GovBizProjectCimplementVO.class);
        cimplementVo.setBaseChanFile(draft.getBaseChanFileCimplement());
        cimplementVo.setBaseEstaSummFile(draft.getBaseEstaSummFileCimplement());
        cimplementVo.setBaseEngineerPostpoFile(draft.getBaseEngineerPostpoFileCimplement());
        cimplementVo.setBaseInforLevelFile(draft.getBaseInforLevelFileCimplement());
        cimplementVo.setBasePasswAssessFile(draft.getBasePasswAssessFileCimplement());
        cimplementVo.setBaseUserConsFile(draft.getBaseUserConsFileCimplement());
        cimplementVo.setBaseThirdAcceptFile(draft.getBaseThirdAcceptFileCimplement());
        cimplementVo.setBaseFinalExpertOpinionFile(draft.getBaseFinalExpertOpinionFileCimplement());
        cimplementVo.setBaseLogAggregation(draft.getBaseLogAggregationCimplement());
        cimplementVo.setBaseBusinessMetrics(draft.getBaseBusinessMetricsCimplement());
        res.setCimplement(cimplementVo);
        GovBizProjectMimplementVO mimplementVo = BeanUtil.copyProperties(draft, GovBizProjectMimplementVO.class);
        mimplementVo.setBaseChanFile(draft.getBaseChanFileMimplement());
        mimplementVo.setBaseEstaSummFile(draft.getBaseEstaSummFileMimplement());
        mimplementVo.setBaseEngineerPostpoFile(draft.getBaseEngineerPostpoFileMimplement());
        mimplementVo.setBaseInforLevelFile(draft.getBaseInforLevelFileMimplement());
        mimplementVo.setBasePasswAssessFile(draft.getBasePasswAssessFileMimplement());
        mimplementVo.setBaseUserConsFile(draft.getBaseUserConsFileMimplement());
        mimplementVo.setBaseThirdAcceptFile(draft.getBaseThirdAcceptFileMimplement());
        mimplementVo.setBaseFinalExpertOpinionFile(draft.getBaseFinalExpertOpinionFileMimplement());
        mimplementVo.setBaseLogAggregation(draft.getBaseLogAggregationMimplement());
        mimplementVo.setBaseBusinessMetrics(draft.getBaseBusinessMetricsMimplement());
        res.setMimplement(mimplementVo);

        //5.最后查询采购信息
        if(StringUtils.isNotBlank(draft.getProcure())){
            try{
                res.setProcures(JSON.parseArray(draft.getProcure(),GovBizProjectProcureVO.class));
            }catch (Exception e){
                log.error("草稿箱采购出错:" + e.getMessage());
            }
        }

        return res;
    }

    /**
     * 项目归集保存
     * @param saveDTO
     * @return
     */
    @Transactional
    public String save(GovBizProjectSaveDTO saveDTO,UserInfoDetails user) {
        if(Objects.isNull(user)){
            user = LoginUserUtil.loginUserDetail();
        }

        GovBizProjectBaseinfoDTO base = saveDTO.getBaseinfo();
        String baseProjId = base.getBaseProjId();
        String baseAreaCode = user.getRegionCode() + BizConst.NINE_AREA_CODE_LAST;
        String baseProjName = base.getBaseProjName();
        //如果是新增的话 要生成 项目编号
        if(StringUtils.isBlank(baseProjId) || BizProjectContant.ProjectCollection.DRAFT.equals(baseProjId)){
            baseProjId = generateProjectCodeUtil.generateProjectCode(saveDTO);
        }

        //1.保存基本信息
        GovBizProjectBaseinfoDTO baseinfo = saveDTO.getBaseinfo();
        baseinfo.setBaseProjId(baseProjId);
        baseinfo.setBaseProjName(baseProjName);
        baseinfo.setIsEffective(BizProjectContant.ProjectCollection.IS_EFFECTIVE);
        GovBizProjectBaseinfo oldBaseInfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, baseProjId)
                .last(BizConst.LIMIT_1));

        //项目名去重
        checkName(baseProjName,baseProjId);
        GovBizProjectBaseinfo saveBase = BeanUtil.copyProperties(baseinfo,GovBizProjectBaseinfo.class);
        saveBase.setBaseProjId(baseProjId);
        saveBase.setBaseProjName(baseProjName);
        saveBase.setIsEffective(BizProjectContant.ProjectCollection.IS_EFFECTIVE);
        if(Objects.nonNull(oldBaseInfo)){
            saveBase.setId(oldBaseInfo.getId());
        }else{
            saveBase.setTongTime(LocalDateTime.now());
            saveBase.setBaseAreaCode(baseAreaCode);
        }
        saveBase.setAreaCode(RegionConst.RC_LS);
        saveBase.setBizTime(LocalDateTime.now());
        saveBase.setUpdateBy(user.getUsername());
        saveBase.setUpdateOn(LocalDateTime.now());
        baseinfoService.saveOrUpdate(saveBase);

        // 2.保存 申报信息
        GovBizProjectApplyDTO apply = saveDTO.getApply();
        if(Objects.nonNull(apply)){
            apply.setBaseProjId(baseProjId);
            apply.setBaseProjName(baseProjName);
            GovBizProjectApply oldApply = projectApplyService.getOne(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .eq(GovBizProjectApply::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectApply saveApply = BeanUtil.copyProperties(apply,GovBizProjectApply.class);
            saveApply.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldApply)){
                saveApply.setId(oldApply.getId());
            }else{
                saveApply.setTongTime(LocalDateTime.now());
            }
            saveApply.setAreaCode(RegionConst.RC_LS);
            saveApply.setBizTime(LocalDateTime.now());
            saveApply.setUpdateBy(user.getUsername());
            saveApply.setUpdateOn(LocalDateTime.now());
            if(projectApplyService.saveOrUpdate(saveApply)){
                uploadFileToProvincialOss(apply,oldApply,saveApply);
            }
        }

        // 3.保存 审批信息
        GovBizProjectApproveDTO approve = saveDTO.getApprove();
        if(Objects.nonNull(approve)){
            approve.setBaseProjId(baseProjId);
            approve.setBaseProjName(baseProjName);
            GovBizProjectApprove oldApprove = approveService.getOne(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                    .eq(GovBizProjectApprove::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectApprove saveApprove = BeanUtil.copyProperties(approve,GovBizProjectApprove.class);
            saveApprove.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldApprove)){
                saveApprove.setId(oldApprove.getId());
            }else{
                saveApprove.setTongTime(LocalDateTime.now());
            }
            saveApprove.setBizTime(LocalDateTime.now());
            saveApprove.setAreaCode(RegionConst.RC_LS);
            saveApprove.setUpdateBy(user.getUsername());
            saveApprove.setUpdateOn(LocalDateTime.now());
            approveService.saveOrUpdate(saveApprove);
        }

        // 4.保存 建设项目 实施信息
        GovBizProjectCimplementDTO cimplement = saveDTO.getCimplement();
        if(Objects.nonNull(cimplement)){
            cimplement.setBaseProjId(baseProjId);
            cimplement.setBaseProjName(baseProjName);
            cimplement.setBaseBidCode(baseProjId + "-1");
            GovBizProjectCimplement oldCimplement = cimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectCimplement.class)
                    .eq(GovBizProjectCimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectCimplement saveCimplement = BeanUtil.copyProperties(cimplement,GovBizProjectCimplement.class);
            saveCimplement.setBaseProjId(baseProjId);
            if(Objects.nonNull(oldCimplement)){
                saveCimplement.setId(oldCimplement.getId());
            }else{
                saveCimplement.setTongTime(LocalDateTime.now());
            }
            saveCimplement.setAreaCode(RegionConst.RC_LS);
            saveCimplement.setBizTime(LocalDateTime.now());
            saveCimplement.setUpdateBy(user.getUsername());
            saveCimplement.setUpdateOn(LocalDateTime.now());
            cimplementService.saveOrUpdate(saveCimplement);
        }

        // 5.保存 运维项目 实施信息
        GovBizProjectMimplementDTO mimplement = saveDTO.getMimplement();
        if(Objects.nonNull(mimplement)){
            mimplement.setBaseProjId(baseProjId);
            mimplement.setBaseProjName(baseProjName);
            mimplement.setBaseBidCode(baseProjId + "-2");
            GovBizProjectMimplement oldMimplement = mimplementService.getOne(Wrappers.lambdaQuery(GovBizProjectMimplement.class)
                    .eq(GovBizProjectMimplement::getBaseProjId, baseProjId)
                    .last(BizConst.LIMIT_1));

            GovBizProjectMimplement saveMimplement = BeanUtil.copyProperties(mimplement,GovBizProjectMimplement.class);
            saveMimplement.setBaseProjId(baseProjId);

            if(Objects.nonNull(oldMimplement)){
                saveMimplement.setId(oldMimplement.getId());
            }else{
                saveMimplement.setTongTime(LocalDateTime.now());
            }
            if(Objects.nonNull(saveBase.getAreaCode())){
                saveMimplement.setAreaCode(saveBase.getAreaCode());
            }
            saveMimplement.setAreaCode(RegionConst.RC_LS);
            saveMimplement.setBizTime(LocalDateTime.now());
            saveMimplement.setUpdateBy(user.getUsername());
            saveMimplement.setUpdateOn(LocalDateTime.now());
            mimplementService.saveOrUpdate(saveMimplement);
        }

        // 6.保存 采购信息
        List<GovBizProjectProcureDTO> procures = saveDTO.getProcures();
        Integer index = 1;
        if(Objects.nonNull(procures)){
            procureService.remove(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                    .eq(GovBizProjectProcure::getBaseProjId,baseProjId));
            for(GovBizProjectProcureDTO procure : procures){
                procure.setBaseProjId(baseProjId);
                procure.setBaseProjName(baseProjName);
                procure.setBaseBidCode(baseProjId + StrPool.DASH + index);

                GovBizProjectProcure saveProcure = BeanUtil.copyProperties(procure,GovBizProjectProcure.class);
                saveProcure.setBaseProjId(baseProjId);
                saveProcure.setId(null);
                saveProcure.setAreaCode(RegionConst.RC_LS);
                saveProcure.setTongTime(LocalDateTime.now());
                saveProcure.setBizTime(LocalDateTime.now());
                saveProcure.setUpdateBy(user.getUsername());
                saveProcure.setUpdateOn(LocalDateTime.now());
                procureService.save(saveProcure);
                index++;
            }
        }

        //7. 推送数据（改为定时推送了）
//        pushProject(saveDTO);
        //如果是草稿箱来的 就要删除
        if(Objects.nonNull(saveDTO.getDraftId())){
            draftService.removeById(saveDTO.getDraftId());
        }

        return BizConst.SAVE_SUCCESS;
    }

    /**
     * 暂存草稿箱
     * @param saveDTO
     * @param user
     * @return
     */
    public String draft(GovBizProjectSaveDTO saveDTO, UserInfoDetails user) {
        if(Objects.isNull(user)){
            user = LoginUserUtil.loginUserDetail();
        }

        GovBizProjectDraft draft = new GovBizProjectDraft();
        //草稿id
        Long draftId = saveDTO.getDraftId();
        String baseAreaCode = user.getRegionCode() + BizConst.NINE_AREA_CODE_LAST;
        if(Objects.nonNull(draftId)){
            GovBizProjectDraft oldDraft = draftService.getById(draftId);
            VUtils.isTrue(Objects.isNull(oldDraft)).throwMessage("该草稿不存在!");
            draft.setId(draftId);
        }else{
            draft.setTongTime(LocalDateTime.now());
            draft.setBaseAreaCode(baseAreaCode);
            draft.setCreateId(user.getUserId());
            draft.setTongTime(LocalDateTime.now());
        }

        //1.保存基本信息
        GovBizProjectBaseinfoDTO base = saveDTO.getBaseinfo();
        String baseProjId = base.getBaseProjId();
        String baseProjName = base.getBaseProjName();
        //要生成 草稿的项目编号
        if(StringUtils.isBlank(baseProjId)){
            baseProjId = BizProjectContant.ProjectCollection.DRAFT;
        }

        // 2.保存 申报信息
        GovBizProjectApplyDTO apply = saveDTO.getApply();
        BeanUtil.copyProperties(apply, draft, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));

        // 3.保存 审批信息
        GovBizProjectApproveDTO approve = saveDTO.getApprove();
        BeanUtil.copyProperties(approve, draft, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));

        // 4.保存 建设项目 实施信息
        GovBizProjectCimplementDTO cimplement = saveDTO.getCimplement();
        if(Objects.nonNull(cimplement)){
            BeanUtil.copyProperties(cimplement, draft, CopyOptions.create()
                    .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
            draft.setBaseBidCode(baseProjId + "-1");
            draft.setBaseChanFileCimplement(cimplement.getBaseChanFile());
            draft.setBaseLogAggregationCimplement(cimplement.getBaseLogAggregation());
            draft.setBaseBusinessMetricsCimplement(cimplement.getBaseBusinessMetrics());
            draft.setBaseEstaSummFileCimplement(cimplement.getBaseEstaSummFile());
            draft.setBaseInforLevelFileCimplement(cimplement.getBaseInforLevelFile());
            draft.setBaseEngineerPostpoFileCimplement(cimplement.getBaseEngineerPostpoFile());
            draft.setBaseUserConsFileCimplement(cimplement.getBaseUserConsFile());
            draft.setBasePasswAssessFileCimplement(cimplement.getBasePasswAssessFile());
            draft.setBaseThirdAcceptFileCimplement(cimplement.getBaseThirdAcceptFile());
            draft.setBaseFinalExpertOpinionFileCimplement(cimplement.getBaseFinalExpertOpinionFile());
        }

        // 5.保存 运维项目 实施信息
        GovBizProjectMimplementDTO mimplement = saveDTO.getMimplement();
        if(Objects.nonNull(mimplement)){
            BeanUtil.copyProperties(mimplement, draft, CopyOptions.create()
                    .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
            draft.setBaseLogAggregationMimplement(mimplement.getBaseLogAggregation());
            draft.setBaseBusinessMetricsMimplement(mimplement.getBaseBusinessMetrics());
            draft.setBaseChanFileMimplement(mimplement.getBaseChanFile());
            draft.setBaseEstaSummFileMimplement(mimplement.getBaseEstaSummFile());
            draft.setBaseInforLevelFileMimplement(mimplement.getBaseInforLevelFile());
            draft.setBaseEngineerPostpoFileMimplement(mimplement.getBaseEngineerPostpoFile());
            draft.setBaseThirdAcceptFileMimplement(mimplement.getBaseThirdAcceptFile());
            draft.setBaseUserConsFileMimplement(mimplement.getBaseUserConsFile());
            draft.setBaseFinalExpertOpinionFileCimplement(mimplement.getBaseFinalExpertOpinionFile());

        }

        // 6.保存 采购信息
        List<GovBizProjectProcureDTO> procures = saveDTO.getProcures();
        if(Objects.nonNull(procures)){
            draft.setProcure(JSON.toJSONString(procures));
        }

        //最后再保存 基本信息
        BeanUtil.copyProperties(base, draft, CopyOptions.create()
                .setIgnoreError(Boolean.TRUE).setIgnoreNullValue(Boolean.TRUE));
        draft.setBaseProjId(baseProjId);
        draft.setBaseProjName(baseProjName);
        draft.setIsEffective(BizProjectContant.ProjectCollection.IS_EFFECTIVE);
        draft.setAreaCode(RegionConst.RC_LS);
        draft.setBizTime(LocalDateTime.now());
        draft.setUpdateBy(user.getUsername());
        draft.setUpdateOn(LocalDateTime.now());
        draft.setOp(BizProjectContant.ProjectCollection.OP_INSERT);

        draftService.saveOrUpdate(draft);

        return BizConst.SAVE_SUCCESS;
    }

    /**
     * 删除项目归集
     * @param projId
     * @return
     */
    public String remove(String projId) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String username = user.getUsername();

        GovBizProjectBaseinfo baseinfo = baseinfoService.getOne(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjId, projId)
                .last(BizConst.LIMIT_1));
        
        VUtils.isTrue(Objects.isNull(baseinfo)).throwMessage("项目不存在！");

        //改正逻辑删除
        baseinfo.setDeleted(Boolean.TRUE);
        baseinfo.setUpdateBy(username);
        baseinfo.setUpdateOn(LocalDateTime.now());
        baseinfoService.updateById(baseinfo);
        projectApplyService.update(Wrappers.lambdaUpdate(GovBizProjectApply.class)
                .eq(GovBizProjectApply::getBaseProjId,projId)
                .set(GovBizProjectApply::getDeleted,Boolean.TRUE)
                .set(GovBizProjectApply::getUpdateOn,LocalDateTime.now())
                .set(GovBizProjectApply::getUpdateBy,username));
        approveService.update(Wrappers.lambdaUpdate(GovBizProjectApprove.class)
                .eq(GovBizProjectApprove::getBaseProjId,projId)
                .set(GovBizProjectApprove::getDeleted,Boolean.TRUE)
                .set(GovBizProjectApprove::getUpdateOn,LocalDateTime.now())
                .set(GovBizProjectApprove::getUpdateBy,username));
        cimplementService.update(Wrappers.lambdaUpdate(GovBizProjectCimplement.class)
                .eq(GovBizProjectCimplement::getBaseProjId,projId)
                .set(GovBizProjectCimplement::getDeleted,Boolean.TRUE)
                .set(GovBizProjectCimplement::getUpdateOn,LocalDateTime.now())
                .set(GovBizProjectCimplement::getUpdateBy,username));
        mimplementService.update(Wrappers.lambdaUpdate(GovBizProjectMimplement.class)
                .eq(GovBizProjectMimplement::getBaseProjId,projId)
                .set(GovBizProjectMimplement::getDeleted,Boolean.TRUE)
                .set(GovBizProjectMimplement::getUpdateOn,LocalDateTime.now())
                .set(GovBizProjectMimplement::getUpdateBy,username));
        procureService.update(Wrappers.lambdaUpdate(GovBizProjectProcure.class)
                .eq(GovBizProjectProcure::getBaseProjId,projId)
                .set(GovBizProjectProcure::getDeleted,Boolean.TRUE)
                .set(GovBizProjectProcure::getUpdateOn,LocalDateTime.now())
                .set(GovBizProjectProcure::getUpdateBy,username));
        return BizConst.OP_SUCCESS;
    }

    /**
     * 项目归集 草稿箱删除
     * @param draftId
     * @return
     */
    public String draftRemove(Long draftId) {
        if(draftService.removeById(draftId)){
            return BizConst.OP_SUCCESS;
        }
        return BizConst.OP_FAIL;
    }

    private void checkName(String baseProjName, String baseProjId) {
        VUtils.isTrue(baseinfoService.count(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                .eq(GovBizProjectBaseinfo::getBaseProjName,baseProjName)
                .eq(GovBizProjectBaseinfo::getDeleted,Boolean.FALSE)
                .ne(GovBizProjectBaseinfo::getBaseProjId,baseProjId)) > 0L)
                .throwMessage("项目名重复");
    }

    /**
     * 推送项目
     * @param saveDTO
     */
    public void pushProject(GovBizProjectSaveDTO saveDTO) {
        String url = pushUrl;
        ProjectPushReq req = ProjectConvert.convert(saveDTO);
        try{
            log.info("推送消息体  {} ,{},{}",saveDTO.getBaseProjId(),url, JSON.toJSONString(req));
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            HttpEntity<ProjectPushReq> formEntity = new HttpEntity(req, headers);
            ResponseEntity<String> response = restTemplate.postForEntity(url, formEntity, String.class);
            log.info("推送结果  {} :{}",saveDTO.getBaseProjId(),response);
        }catch (Exception e){
            log.info("推送失败,{}",e.getMessage());
        }
    }
    public void pushProjectVo(GovBizProjectDetailVO vo) {
        String url = pushUrl;
        ProjectPushReq req = ProjectConvert.convert(vo);
        try{
            log.info("推送消息体  {} ,{},{}",vo.getBaseProjId(),url, JSON.toJSONString(req));
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            HttpEntity<ProjectPushReq> formEntity = new HttpEntity<ProjectPushReq>(req, headers);
            ResponseEntity<String> response = restTemplate.postForEntity(url, formEntity, String.class);
            log.info("推送结果  {} :{}",vo.getBaseProjId(),response);
        }catch (Exception e){
            log.info("推送失败,{}",e.getMessage());
        }
    }

    /**
     * 把项目置为无效
     * @param baseinfo
     */
    private void notEffectiveProject(GovBizProjectBaseinfoDTO baseinfo) {
        String url = noEffectiveUrl;
        ProjectBaseInfoReq req = ProjectConvert.convertBase(baseinfo,BizProjectContant.NOT_EFFECTIVE_CODE);
        try{
            log.info("推送消息体  {},{},{}",baseinfo.getBaseProjId(),url, JSON.toJSONString(req));
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            HttpEntity<ProjectBaseInfoReq> formEntity = new HttpEntity<ProjectBaseInfoReq>(req, headers);
            ResponseEntity<String> response = restTemplate.postForEntity(url, formEntity, String.class);
            log.info("推送结果  {} :{}",baseinfo.getBaseProjId(),response);
        }catch (Exception e){
            log.info("推送失败,{}",e.getMessage());
        }
    }

    public void deleteAll() {
        String url = deleteAllUrl;
        try {
            log.info("推送删除数据url  {}", url);
            HttpHeaders headers = new HttpHeaders();
            MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
            headers.setContentType(type);
            headers.add("Accept", MediaType.APPLICATION_JSON.toString());
            HttpEntity<String> formEntity = new HttpEntity<String>(null, headers);
            ResponseEntity<String> response = restTemplate.postForEntity(url, formEntity, String.class);
            log.info("推送删除结果 :{}", response);
        } catch (Exception e) {
            log.info("推送删除失败,{}", e.getMessage());
        }
    }

    /**
     * 上传项目归集申报的文件到OSS 前提还要先转为PDF
     * @param apply
     * @param oldApply
     */
    private void uploadFileToProvincialOss(GovBizProjectApplyDTO apply,
                                           GovBizProjectApply oldApply,GovBizProjectApply saveApply) {
        if(checkFieldHas(apply.getBaseProjBasisFile(),oldApply,"baseProjBasisFile")){
            JSONArray fileArray = JSON.parseArray(apply.getBaseProjBasisFile());
            StringJoiner sj = new StringJoiner("；");
            fileArray.forEach(j -> {
                JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(j));
                byte[] btyes = wpsConvertManage.downloadToPdfStream(jsonObject.getLong("id"));
                String oss = ProvincialManage.uploadToOss(btyes, jsonObject.getString("originalFileName"));
                sj.add(oss);
            });
            saveApply.setBaseProjBasisFilePdf(sj.toString());
        }
        projectApplyService.updateById(saveApply);
    }

    private Boolean checkFieldHas(String field,GovBizProjectApply oldApply,String fieldName){
        if(StringUtils.isBlank(field)){
            return Boolean.FALSE;
        }

        //如果没有老值
        if(Objects.isNull(oldApply)){
            return Boolean.TRUE;
        }

        //和老值做个对比 不一样则true
        Class<?> clazz = oldApply.getClass();
        String oldFiled = StringUtils.EMPTY;
        try {
            Field fieldDeclared = clazz.getDeclaredField(fieldName);
            fieldDeclared.setAccessible(Boolean.TRUE);
            oldFiled = Objects.nonNull(fieldDeclared.get(oldApply)) ? String.valueOf(fieldDeclared.get(oldApply)) : null;
        } catch (NoSuchFieldException e) {
            log.error("转换PDF文件出错" + e);
        } catch (IllegalAccessException e) {
            log.error("转换PDF文件出错" + e);
        }

        if(!oldFiled.equals(field)){
            return Boolean.TRUE;
        }

        return Boolean.FALSE;
    }
}
