package com.ningdatech.pmapi.performance.manage;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.CharsetUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.performance.constant.BizConst;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalExportReq;
import com.ningdatech.pmapi.performance.util.*;
import com.ningdatech.pmapi.portrait.model.entity.ProjectTag;
import com.ningdatech.pmapi.portrait.service.IProjectTagService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.pmapi.performance.enumration.AppraisalTypeEnum;
import com.ningdatech.pmapi.performance.enumration.RectifyStatusEnum;
import com.ningdatech.pmapi.performance.model.dto.ProjectAppraisalDTO;
import com.ningdatech.pmapi.performance.model.dto.ProjectAppraisalInfoDTO;
import com.ningdatech.pmapi.performance.model.entity.*;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalProjectVO;
import com.ningdatech.pmapi.performance.model.vo.PerformanceAppraisalVO;
import com.ningdatech.pmapi.performance.service.*;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectTypeEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.sys.service.IRoleService;
import com.ningdatech.pmapi.user.entity.enumeration.RoleEnum;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletResponse;

/**
 * VerifyManage
 * @return
 * @author CMM
 * @since 2023/08/10 8:55
 */
@Component
@Slf4j
@AllArgsConstructor
public class VerifyManage {

    private final IPerformanceAppraisalService performanceAppraisalService;
    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;
    private final IProjectService projectService;
    private final IRoleService roleService;
    private final IPerformanceIndicatorProjectTemplateDetailService indicatorProjectTemplateDetailService;
    private final IPerformanceAppraisalScoreInfoService performanceAppraisalScoreInfoService;
    private final IPerformanceAppraisalProjectGroupService groupService;
    private final IProjectTagService projectTagService;
    private final IPerformanceIndicatorProjectTemplateService indicatorProjectTemplateService;
    /**
     * 核查-评价计划列表
     * @param req
     * @return
     */
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String employeeCode = user.getEmployeeCode();
        // 获取当前登录用户的角色列表，只有专家核查人员才能查看该列表
        List<Role> userRoleList = user.getUserRoleList();
        List<Long> roleIdList = userRoleList.stream().map(Role::getId).collect(Collectors.toList());
        Role role = roleService.getOne(Wrappers.lambdaQuery(Role.class)
                .eq(Role::getName, RoleEnum.EXPERT.getDesc()));
        if (Objects.isNull(role)){
            throw new BizException("登录用户没有复评员角色!");
        }

        // 登录用户不是专家，不能查看核查计划列表
        if (!roleIdList.contains(role.getId())){
            return PageVo.empty();
        }

        // 从绩效评价 分组表中获取当前登录专家所在的分组信息
        List<PerformanceAppraisalProjectGroup> groups = Lists.newArrayList();
        List<PerformanceAppraisalProjectGroup> projectGroups = groupService.list();
        for (PerformanceAppraisalProjectGroup projectGroup : projectGroups) {
            String verificationUsers = projectGroup.getVerificationUsers();
            if (verificationUsers.contains(StrPool.COMMA)){
                if (verificationUsers.contains(employeeCode)){
                    groups.add(projectGroup);
                }
            } else if (verificationUsers.equals(employeeCode)) {
                groups.add(projectGroup);
            }
        }
        // 当前登录专家没有绩效分组信息
        if (CollUtil.isEmpty(groups)){
            return PageVo.empty();
        }

        // 获取分组所在的评价计划ID
        List<Long> appraisalIds = groups.stream().map(PerformanceAppraisalProjectGroup::getAppraisalId).collect(Collectors.toList());
        // 因为同一个核查人员只能出现在一个计划的一个分组中，所以分组信息可按照评价计划ID进行分组
        Map<Long, PerformanceAppraisalProjectGroup> groupMap = groups.stream().collect(Collectors.toMap(PerformanceAppraisalProjectGroup::getAppraisalId, g -> g));
        // 获取核查计划列表
        Page<PerformanceAppraisal> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisal> wrapper = Wrappers.lambdaQuery(PerformanceAppraisal.class)
                .in(PerformanceAppraisal::getId,appraisalIds)
                .orderByDesc(PerformanceAppraisal::getUpdateOn);
        performanceAppraisalService.page(page,wrapper);

        List<PerformanceAppraisalVO> res = page.getRecords().stream().map(p -> {
            PerformanceAppraisalVO vo = BeanUtil.copyProperties(p, PerformanceAppraisalVO.class);
            PerformanceAppraisalProjectGroup group = groupMap.get(p.getId());
            if (Objects.nonNull(group)) {
                vo.setVerifyGroupId(group.getId());
                vo.setVerifyGroupName(group.getName());
            }
            return vo;
        }).collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }

    /**
     * 是否可以核查
     *
     * @param employeeCode
     * @param projectCode
     * @param appraisalId
     * @return
     */
    private Boolean checkCanVerify(String employeeCode, String projectCode, Long appraisalId) {

        // 首次核查(核查打分信息为空)
        // 从绩效评价打分信息表中查询打分类型为专家核查、打分人员为当前登录专家用户的打分信息
        // 如果没有相关的打分信息，说明是当前登录复评用户是首次核查
        List<PerformanceAppraisalScoreInfo> scoreInfos = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.EXPERT_VERIFY.getCode())
                .eq(PerformanceAppraisalScoreInfo::getAppraisalId,appraisalId)
                .eq(PerformanceAppraisalScoreInfo::getAppraisalProjectCode,projectCode)
                .eq(PerformanceAppraisalScoreInfo::getAppraisalEmployeeCode, employeeCode));
        if (CollUtil.isEmpty(scoreInfos)){
            return true;
        }
        return Boolean.FALSE;
    }

    public PageVo<PerformanceAppraisalProjectVO> groupProjectList(Long groupId, PerformanceAppraisalListReq req) {

        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        String employeeCode = userDetail.getEmployeeCode();

        // 获取分组内的项目信息
        PerformanceAppraisalProjectGroup projectGroup = groupService.getById(groupId);
        if (Objects.isNull(projectGroup)){
            throw new BizException("当前核查用户所在分组不存在!");
        }
        String projectCodes = projectGroup.getProjectCodes();
        Long appraisalId = projectGroup.getAppraisalId();
        VUtils.isTrue(StringUtils.isBlank(projectCodes)).throwMessage("分组内项目信息不存在!");
        List<String> projectCodeList = Arrays.stream(projectCodes.split(StrPool.COMMA)).collect(Collectors.toList());

        // 筛选出分组内已完成复评的评价项目信息
        List<PerformanceAppraisalProject> appraisalProjects = performanceAppraisalProjectService.list(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .in(PerformanceAppraisalProject::getProjectCode, projectCodeList)
                .eq(PerformanceAppraisalProject::getIsCompleteReAppraisal, Boolean.TRUE));
        // 分组内已完成复评的项目信息不存在
        if (CollUtil.isEmpty(appraisalProjects)){
            return PageVo.empty();
        }
        List<String> proCodeList = appraisalProjects.stream().map(PerformanceAppraisalProject::getProjectCode).collect(Collectors.toList());
        Map<String, PerformanceAppraisalProject> appraisalProjectMap = appraisalProjects.stream().collect(Collectors.toMap(PerformanceAppraisalProject::getProjectCode, p -> p));

        Page<Project> page = req.page();
        LambdaQueryWrapper<Project> wrapper = Wrappers.lambdaQuery(Project.class)
                .in(Project::getProjectCode, proCodeList)
                .eq(Project::getNewest, Boolean.TRUE)
                .like(StringUtils.isNotBlank(req.getProjectName()),Project::getProjectName,req.getProjectName());
        projectService.page(page,wrapper);

        // 分组内项目信息不存在
        if (CollUtil.isEmpty(page.getRecords())){
            return PageVo.empty();
        }

        List<PerformanceAppraisalProjectVO> res = page.getRecords().stream().map(p -> {
                    PerformanceAppraisalProjectVO vo = BeanUtil.copyProperties(p, PerformanceAppraisalProjectVO.class);
                    PerformanceAppraisalProject appraisalProject = appraisalProjectMap.get(p.getProjectCode());
                    BeanUtil.copyProperties(appraisalProject, vo);
                    vo.setProjectId(appraisalProject.getProjectId());
                    vo.setProjectTypeName(ProjectTypeEnum.getDesc(p.getProjectType()));
                    vo.setCanVerify(checkCanVerify(employeeCode,appraisalProject.getProjectCode(),appraisalId));
                    // 如果登录用户是核查组长，该项目可填写整改意见
                    String verifyLeader = projectGroup.getVerifyLeader();
                    if (employeeCode.equals(verifyLeader)){
                        vo.setCanRectify(Boolean.TRUE);
                    }
                    vo.setGroupId(groupId);
                    return vo;
                })
                .collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }

    @Transactional(rollbackFor = Exception.class)
    public String submitVerify(ProjectAppraisalDTO param) {
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        String employeeCode = userDetail.getEmployeeCode();
        String username = userDetail.getUsername();
        Long appraisalId = param.getAppraisalId();
        Long appraisalProjectId = param.getAppraisalProjectId();
        String appraisalProjectCode = param.getAppraisalProjectCode();
        List<ProjectAppraisalInfoDTO> appraisalInfoList = param.getAppraisalInfoList();
        Long groupId = param.getGroupId();
        PerformanceAppraisalProjectGroup group = groupService.getById(groupId);

        // 获取评价项目
        PerformanceAppraisalProject appraisalProject = performanceAppraisalProjectService.getOne(Wrappers.lambdaQuery(PerformanceAppraisalProject.class)
                .eq(PerformanceAppraisalProject::getAppraisalId, appraisalId)
                .eq(PerformanceAppraisalProject::getProjectCode,appraisalProjectCode)
                .eq(PerformanceAppraisalProject::getProjectId, appraisalProjectId));

        List<Long> ids = appraisalInfoList.stream()
                .map(ProjectAppraisalInfoDTO::getProjectTemplateDetailId)
                .collect(Collectors.toList());
        // 查出项目模版详情信息
        List<PerformanceIndicatorProjectTemplateDetail> detailList = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                .in(PerformanceIndicatorProjectTemplateDetail::getId, ids));
        if (CollUtil.isEmpty(detailList)){
            throw new BizException("评价指标不存在!");
        }
        Map<Long, PerformanceIndicatorProjectTemplateDetail> detailMap = detailList.stream().collect(Collectors.toMap(PerformanceIndicatorProjectTemplateDetail::getId, d -> d));
        List<PerformanceAppraisalScoreInfo> scoreInfos = appraisalInfoList.stream().map(a -> {
            PerformanceIndicatorProjectTemplateDetail templateDetail = detailMap.get(a.getProjectTemplateDetailId());
            BigDecimal score = a.getScore();
            BigDecimal indexScore = templateDetail.getIndexScore();
            if (score.compareTo(indexScore) > 0 || score.compareTo(BigDecimal.ZERO) < 0){
                throw new BizException("核查得分必须大于等于0且小于等于指标分值!");
            }
            // 保存核查分数信息
            PerformanceAppraisalScoreInfo scoreInfo = new PerformanceAppraisalScoreInfo();
            scoreInfo.setAppraisalBasis(a.getAppraisalBasis());
            scoreInfo.setAppraisalScore(a.getScore());
            scoreInfo.setAppraisalBasisAppendix(a.getAppraisalBasisAppendix());
            scoreInfo.setAppraisalType(AppraisalTypeEnum.EXPERT_VERIFY.getCode());
            scoreInfo.setTemplateDetailId(templateDetail.getId());
            scoreInfo.setTemplateId(templateDetail.getTemplateId());
            scoreInfo.setAppraisalEmployeeCode(employeeCode);
            scoreInfo.setAppraisalEmployeeName(username);
            scoreInfo.setAppraisalId(appraisalId);
            scoreInfo.setAppraisalProjectId(appraisalProjectId);
            scoreInfo.setAppraisalProjectCode(appraisalProjectCode);
            scoreInfo.setCreateOn(LocalDateTime.now());
            scoreInfo.setCreateBy(username);
            return scoreInfo;
        }).collect(Collectors.toList());

        if (performanceAppraisalScoreInfoService.saveBatch(scoreInfos)){
            // 计算核查总分并保存
            BigDecimal verifyTotalScore = appraisalInfoList.stream()
                    .map(ProjectAppraisalInfoDTO::getScore)
                    .filter(Objects::nonNull)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal lastVerifyTotalScore;
            // 判断项目所在分组内其他复评人员是否已经打分
            if (Objects.nonNull(appraisalProject.getVerifyTotalScore())){
                // 已经进行了核查，计算两个分值的平均分
                BigDecimal add = verifyTotalScore.add(appraisalProject.getReAppraisalTotalScore());
                lastVerifyTotalScore = add.divide(BigDecimal.valueOf(2));
            }else {
                // 还没有进行过复评
                lastVerifyTotalScore = verifyTotalScore;
            }
            appraisalProject.setVerifyTotalScore(lastVerifyTotalScore);
            // 取最新地核查打分时间
            appraisalProject.setVerifyScoreTime(LocalDateTime.now());
            // 判断当前核查人是否为该评价项目所在分组的核查人员中的最后一个核查人
            judgeCompleteVerify(group, appraisalProject);
            appraisalProject.setUpdateOn(LocalDateTime.now());
            appraisalProject.setUpdateBy(username);
            // 保存整改相关信息
            if (Objects.nonNull(param.getIsRectify()) ||
                    StringUtils.isNotBlank(param.getRectifyOpinion())) {
                appraisalProject.setIsRectify(param.getIsRectify());
                appraisalProject.setRectifyOpinion(param.getRectifyOpinion());
                appraisalProject.setRectifySubmitEmployeeCode(employeeCode);
                appraisalProject.setRectifySubmitTime(LocalDateTime.now());
                appraisalProject.setRectifyStatus(RectifyStatusEnum.TO_BE_SUBMITTED.getCode());
            }
            performanceAppraisalProjectService.updateById(appraisalProject);
            return "提交成功";
        }
        return "提交失败";
    }

    private void judgeCompleteVerify(PerformanceAppraisalProjectGroup group, PerformanceAppraisalProject appraisalProject) {
        // 获取该评价计划当前分组对该项目的核查打分信息
        List<PerformanceAppraisalScoreInfo> scoreInfoList = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                        .eq(PerformanceAppraisalScoreInfo::getAppraisalId,group.getAppraisalId())
                        .eq(PerformanceAppraisalScoreInfo::getAppraisalProjectCode,appraisalProject.getProjectCode())
                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.EXPERT_VERIFY.getCode()));

        // 判断打分人员是否为分组内的核查人员
        if (CollUtil.isNotEmpty(scoreInfoList)) {
            Set<String> users = scoreInfoList.stream().map(PerformanceAppraisalScoreInfo::getAppraisalEmployeeCode).collect(Collectors.toSet());
            if (CollUtil.isNotEmpty(users)) {
                String verificationUsers = group.getVerificationUsers();
                String join = String.join(StrPool.COMMA, users);
                if (join.equals(verificationUsers)) {
                    // 标记项目完成核查
                    appraisalProject.setIsCompleteVerify(Boolean.TRUE);
                }
            }
        }
    }

    public void exportScore(HttpServletResponse response, PerformanceAppraisalExportReq param) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String regionCode = user.getRegionCode();
        String projectCode = param.getProjectCode();
        Long appraisalId = param.getAppraisalId();

        // 根据项目编码获取最新版本的项目信息
        Project project = projectService.getProjectByCode(projectCode);
        VUtils.isTrue(Objects.isNull(project)).throwMessage("项目不存在!");
        // 根据项目类型、预算年度、批复金额、项目标签匹配本区域指标模版
        Integer projectType = project.getProjectType();
        Integer projectYear = project.getProjectYear();
        BigDecimal approvalAmount = project.getApprovalAmount();
        VUtils.isTrue(Objects.isNull(approvalAmount)).throwMessage("未获取到该项目的立项批复金额");
        // 根据项目code获取项目标签ID列表
        List<Long> tagIdList = projectTagService.list(Wrappers.lambdaQuery(ProjectTag.class)
                        .eq(ProjectTag::getProjectCode, projectCode)).stream()
                .map(ProjectTag::getTagId).collect(Collectors.toList());
        VUtils.isTrue(CollUtil.isEmpty(tagIdList)).throwMessage("当前项目未设置标签,匹配不到指标模板,请至项目库或评价计划编辑页面设置标签!");

        List<String> strIdList = tagIdList.stream().map(String::valueOf).collect(Collectors.toList());
        String projectTagIds = String.join(StrPool.COMMA, strIdList);

        Integer amountRange = null;
        if (approvalAmount.compareTo(BigDecimal.valueOf(BizConst.FIVE_MILLION)) < 0){
            amountRange = BizConst.AMOUNT_RANGE_ONE;
        } else if (approvalAmount.compareTo(BigDecimal.valueOf(BizConst.FIVE_MILLION)) >= 0 &&
                approvalAmount.compareTo(BigDecimal.valueOf(BizConst.TWENTY_MILLION)) < 0) {
            amountRange = BizConst.AMOUNT_RANGE_TWO;
        }else if (approvalAmount.compareTo(BigDecimal.valueOf(BizConst.TWENTY_MILLION)) >= 0){
            amountRange = BizConst.AMOUNT_RANGE_THREE;
        }
        LambdaQueryWrapper<PerformanceIndicatorProjectTemplate> wrapper = Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplate.class)
                .eq(PerformanceIndicatorProjectTemplate::getRegionCode, regionCode)
                .eq(PerformanceIndicatorProjectTemplate::getProjectType, projectType)
                .eq(PerformanceIndicatorProjectTemplate::getProjectYear, projectYear)
                .eq(PerformanceIndicatorProjectTemplate::getAmountRange, amountRange)
                .eq(PerformanceIndicatorProjectTemplate::getProjectTagIds,projectTagIds)
                .eq(PerformanceIndicatorProjectTemplate::getStatus, CommonEnum.YES.getCode())
                .orderByDesc(PerformanceIndicatorProjectTemplate::getUpdateOn);
        List<PerformanceIndicatorProjectTemplate> templates = indicatorProjectTemplateService.list(wrapper);
        VUtils.isTrue(CollUtil.isEmpty(templates)).throwMessage("该项目匹配不到指标模板，请返回上一页或者刷新重试。");
        if (templates.size() > 1){
            throw new BizException("当前项目匹配到多个模板，请返回模板库检查模板配置!");
        }
        PerformanceIndicatorProjectTemplate projectTemplate = templates.get(0);
        // 装配项目指标详情及分数信息
        // 获取模版绩效指标详情
        List<PerformanceIndicatorProjectTemplateDetail> templateDetails = indicatorProjectTemplateDetailService.list(Wrappers.lambdaQuery(PerformanceIndicatorProjectTemplateDetail.class)
                .eq(PerformanceIndicatorProjectTemplateDetail::getTemplateId, projectTemplate.getId()));
        Map<Long, PerformanceIndicatorProjectTemplateDetail> templateDetailMap = templateDetails.stream().collect(Collectors.toMap(PerformanceIndicatorProjectTemplateDetail::getId, t -> t));

        List<Map<String, Object>> exportDataList = Lists.newArrayList();
        List<Map<String, Object>> exportAdditionalDataList = Lists.newArrayList();
        // 构建指标和打分详情
        List<PerformanceAppraisalScoreInfo> scoreInfoList = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                .eq(PerformanceAppraisalScoreInfo::getAppraisalId, appraisalId)
                .and(wp -> wp.eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.SELF_APPRAISAL.getCode()).or()
                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.RE_APPRAISAL.getCode()).or()
                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.EXPERT_VERIFY.getCode()))
                .eq(PerformanceAppraisalScoreInfo::getAppraisalProjectCode, projectCode)
                .orderBy(Boolean.TRUE, Boolean.TRUE, PerformanceAppraisalScoreInfo::getTemplateDetailId));
        Map<Long, PerformanceAppraisalScoreInfo> scoreInfoMap = scoreInfoList.stream()
                .collect(Collectors.toMap(PerformanceAppraisalScoreInfo::getTemplateDetailId, p -> p));

        // 获取复评打分人员信息
        List<PerformanceAppraisalScoreInfo> reScoreInfoList = scoreInfoList.stream()
                .filter(s -> AppraisalTypeEnum.RE_APPRAISAL.getCode().equals(s.getAppraisalType()))
                .collect(Collectors.toList());
        List<String> reUserList = reScoreInfoList.stream()
                .map(PerformanceAppraisalScoreInfo::getAppraisalEmployeeName)
                .collect(Collectors.toList());

        // 获取核查打分人员信息
        List<PerformanceAppraisalScoreInfo> verifyScoreInfoList = scoreInfoList.stream()
                .filter(s -> AppraisalTypeEnum.EXPERT_VERIFY.getCode().equals(s.getAppraisalType()))
                .collect(Collectors.toList());
        List<String> verifyUserList = verifyScoreInfoList.stream()
                .map(PerformanceAppraisalScoreInfo::getAppraisalEmployeeName)
                .collect(Collectors.toList());

        // 筛选出所有打分的三级指标模板详情ID
        for (PerformanceAppraisalScoreInfo scoreInfo : scoreInfoList) {
            HashMap<String, Object> scoreMap = MapUtil.newHashMap();
            Long detailId = scoreInfo.getTemplateDetailId();
            PerformanceIndicatorProjectTemplateDetail thirdTemplateDetail = templateDetailMap.get(detailId);

            // 二级指标名称
            Long secondId = thirdTemplateDetail.getParentId();
            if (Objects.nonNull(secondId)) {
                PerformanceIndicatorProjectTemplateDetail secondTemplateDetail = templateDetailMap.get(secondId);
                if (Objects.nonNull(secondTemplateDetail)) {
                    // 一级指标名称
                    Long firstId = secondTemplateDetail.getParentId();
                    if (Objects.nonNull(firstId)){
                        PerformanceIndicatorProjectTemplateDetail firstTemplateDetail = templateDetailMap.get(firstId);
                        if (Objects.nonNull(firstTemplateDetail)) {
                            scoreMap.put(BizConst.FIRST_INDEX_NAME,firstTemplateDetail.getName());
                            scoreMap.put(BizConst.SECOND_INDEX_NAME,secondTemplateDetail.getName());
                        }
                    }
                }
            }
            scoreMap.put(BizConst.THIRD_INDEX_NAME,thirdTemplateDetail.getName());
            scoreMap.put(BizConst.INDEX_SCORE,thirdTemplateDetail.getIndexScore());
            scoreMap.put(BizConst.INDEX_DETAIL,thirdTemplateDetail.getIndexDetail());
            scoreMap.put(BizConst.GRADE_DETAIL,thirdTemplateDetail.getGradeDetail());
            scoreMap.put(BizConst.SUPPORT_MATERIAL,thirdTemplateDetail.getSupportMaterial());

            PerformanceAppraisalScoreInfo appraisalScoreInfo = scoreInfoMap.get(thirdTemplateDetail.getId());
            // 构建自评和复评打分信息
            if (AppraisalTypeEnum.SELF_APPRAISAL.getCode().equals(appraisalScoreInfo.getAppraisalType())){
                scoreMap.put(BizConst.SELF_APPRAISAL_BASIS,appraisalScoreInfo.getAppraisalBasis());
                scoreMap.put(BizConst.SELF_APPRAISAL_SCORE,appraisalScoreInfo.getAppraisalScore());
            } else if (AppraisalTypeEnum.RE_APPRAISAL.getCode().equals(appraisalScoreInfo.getAppraisalType())) {
                scoreMap.put(appraisalScoreInfo.getAppraisalEmployeeName() + BizConst.RE_APPRAISAL_BASIS, appraisalScoreInfo.getAppraisalBasis());
                scoreMap.put(appraisalScoreInfo.getAppraisalEmployeeName() + BizConst.RE_APPRAISAL_SCORE, appraisalScoreInfo.getAppraisalScore());
            } else if (AppraisalTypeEnum.EXPERT_VERIFY.getCode().equals(appraisalScoreInfo.getAppraisalType())) {
                scoreMap.put(appraisalScoreInfo.getAppraisalEmployeeName() + BizConst.VERIFY_BASIS, appraisalScoreInfo.getAppraisalBasis());
                scoreMap.put(appraisalScoreInfo.getAppraisalEmployeeName() + BizConst.VERIFY_SCORE, appraisalScoreInfo.getAppraisalScore());
            }

            if (Boolean.TRUE.equals(thirdTemplateDetail.getIsAdditional())) {
                scoreMap.put(BizConst.IS_ADDITIONAL,CommonEnum.YES.getDesc());
            } else if (Boolean.FALSE.equals(thirdTemplateDetail.getIsAdditional())) {
                scoreMap.put(BizConst.IS_ADDITIONAL,CommonEnum.NO.getDesc());
            }
            if (Boolean.FALSE.equals(thirdTemplateDetail.getIsAdditional())) {
                exportDataList.add(scoreMap);
            }else {
                exportAdditionalDataList.add(scoreMap);
            }
        }
        exportDataList.addAll(exportAdditionalDataList);
        String fileName = "绩效评价_专家核查_评分明细表";
        ExcelDownUtil.setFileName(fileName, response);
        int[] mergeColumnIndex = {0};
        // 需要从第几行开始合并
        int mergeRowIndex = 2;

        List<ExcelHead> headList = Lists.newArrayList();
        headList.add(new ExcelHead(BizConst.FIRST_INDEX_NAME,BizConst.FIRST_INDEX_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.SECOND_INDEX_NAME,BizConst.SECOND_INDEX_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.THIRD_INDEX_NAME,BizConst.THIRD_INDEX_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.INDEX_SCORE,BizConst.INDEX_SCORE_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.INDEX_DETAIL,BizConst.INDEX_DETAIL_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.GRADE_DETAIL,BizConst.GRADE_DETAIL_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.SUPPORT_MATERIAL,BizConst.SUPPORT_MATERIAL_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.SELF_APPRAISAL_BASIS,BizConst.SELF_APPRAISAL_BASIS_TITLE,StrPool.EMPTY));
        headList.add(new ExcelHead(BizConst.SELF_APPRAISAL_SCORE,BizConst.SELF_APPRAISAL_SCORE_TITLE,StrPool.EMPTY));
        for (String reUser : reUserList) {
            headList.add(new ExcelHead(reUser + BizConst.RE_APPRAISAL_BASIS,BizConst.RE_APPRAISAL_BASIS_TITLE,StrPool.EMPTY));
            headList.add(new ExcelHead(reUser + BizConst.RE_APPRAISAL_SCORE,BizConst.RE_APPRAISAL_SCORE_TITLE,StrPool.EMPTY));
        }

        for (String verifyUser : verifyUserList) {
            headList.add(new ExcelHead(verifyUser + BizConst.VERIFY_BASIS,BizConst.VERIFY_BASIS_TITLE,StrPool.EMPTY));
            headList.add(new ExcelHead(verifyUser + BizConst.VERIFY_SCORE,BizConst.VERIFY_SCORE_TITLE,StrPool.EMPTY));
        }
        headList.add(new ExcelHead(BizConst.IS_ADDITIONAL,BizConst.IS_ADDITIONAL_TITLE,StrPool.EMPTY));

        // 数据导出处理函数
        try {
            response.setCharacterEncoding(CharsetUtil.UTF_8);
            response.setContentType(ContentTypeUtils.APPLICATION_EXCEL);
            ExcelWriterBuilder writerBuilder = EasyExcel.write();
            writerBuilder.file(response.getOutputStream());
            writerBuilder.autoCloseStream(true);
            writerBuilder.registerWriteHandler(new ExcelSheetVerticalCellStyleStrategy());
            writerBuilder.registerWriteHandler(new MultiColumnMergeStrategy(exportDataList.size(),0,1));
            writerBuilder.registerWriteHandler(new ExcelFillCellMergeStrategy(mergeRowIndex,mergeColumnIndex));
            writerBuilder.head(ExcelUtils.getHead(reUserList,verifyUserList))
                    .sheet(fileName)
                    .doWrite(ExcelUtils.convertData(headList, exportDataList));
        } catch (IOException e) {
            throw new BizException(e.getMessage());
        }
    }

}
