package com.ningdatech.pmapi.projectdeclared.utils;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.gov.contants.BizProjectContant;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectApplyDTO;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectBaseinfoDTO;
import com.ningdatech.pmapi.gov.model.dto.GovBizProjectSaveDTO;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectApply;
import com.ningdatech.pmapi.gov.service.IGovBizProjectApplyService;
import com.ningdatech.pmapi.projectdeclared.contants.ProjectCodeContant;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectCollection;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * 生成项目编号工具类
 *
 * @author CMM
 * @since 2023/04/04 14:13
 */
@Component
public class GenerateProjectCodeUtil {

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IGovBizProjectApplyService applyService;

    public String generateProjectCode(ProjectDTO project){
        // 获取所属行政区划代码(9位)
        String areaCode = project.getAreaCode() + ProjectCodeContant.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType =  ProjectCodeContant.PROJECT_TYPE_PREFIX + project.getProjectType();
        // 16-20 项目序号00001
        Long max = projectService.count(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectYear,project.getProjectYear())
                .eq(Project::getNewest,Boolean.TRUE));
        Long maxCurrent = max + 1;
        String serialNumber = String.format(ProjectCodeContant.SHUZI_5,maxCurrent);
        // 生成20位的项目编号
        return areaCode + year + projectType + serialNumber;
    }

    public String generateProjectCode(ProjectCollection project){
        // 获取所属行政区划代码(9位)
        String areaCode = project.getAreaCode() + ProjectCodeContant.REGION_LAST_3;
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(project.getProjectYear()) ? String.valueOf(project.getProjectYear())
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType =  ProjectCodeContant.PROJECT_TYPE_PREFIX + project.getProjectType();
        // 16-20 项目序号00001
        Long max = projectService.count(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectYear,project.getProjectYear())
                .eq(Project::getNewest,Boolean.TRUE));
        Long maxCurrent = max + 1;
        String serialNumber = String.format(ProjectCodeContant.SHUZI_5,maxCurrent);
        // 生成20位的项目编号
        return areaCode + year + projectType + serialNumber;
    }

    public String generateProjectCode(GovBizProjectSaveDTO dto){
        // 获取所属行政区划代码(9位)
        GovBizProjectBaseinfoDTO baseinfo = dto.getBaseinfo();
        GovBizProjectApplyDTO apply = dto.getApply();

        VUtils.isTrue(Objects.isNull(baseinfo)).throwMessage("没有项目基本信息 无法生成项目编号!");
        VUtils.isTrue(Objects.isNull(apply)).throwMessage("没有项目申报信息 无法生成项目编号!");

        String areaCode = (StringUtils.isNotBlank(baseinfo.getBaseAreaCode())?baseinfo.getBaseAreaCode(): RegionConst.RC_LS + BizConst.NINE_AREA_CODE_LAST);
        // 获取建设年度 (10-13)
        String year = Objects.nonNull(apply.getBaseProjSetYear()) ? apply.getBaseProjSetYear()
                : String.valueOf(LocalDateTime.now().getYear());
        // 14-15 项目类型
        String projectType = StringUtils.isNotBlank(baseinfo.getBaseProjType()) ? baseinfo.getBaseProjType() :
                BizProjectContant.ProjectCollection.IS_EFFECTIVE;
        // 16-20 项目序号00001
        Long max = applyService.count(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .eq(GovBizProjectApply::getBaseProjSetYear,apply.getBaseProjSetYear()));
        Long maxCurrent = max + 1;
        String serialNumber = String.format(ProjectCodeContant.SHUZI_5,maxCurrent);
        // 生成20位的项目编号
        return areaCode + year + projectType + serialNumber;
    }
}
