package com.ningdatech.pmapi.provincial.model.res;

import com.ningdatech.basic.enumeration.Status;
import com.ningdatech.basic.model.ApiStatus;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;

/**
 * <p>
 * OssApiResponse -
 * </p>
 *
 * @author ZPF
 * @since 14:29 2022/9/29
 */
@Data
@NoArgsConstructor
public class OssApiResponse<T> implements Serializable {
    private static final long serialVersionUID = 1L;

    public static final int SUCCESS_CODE = 200;

    public static final String SUCCESS_MSG = "成功";

    public static final int ERROR_CODE = 500;

    public static final String ERROR_MSG = "Internal server error";

    /**
     * 状态码
     */
    private Integer respCode;

    /**
     * 返回描述
     */
    private String respMsg;

    /**
     * 返回数据
     */
    private T data;

    /**
     * 全参构造函数
     *
     * @param respCode    状态码
     * @param respMsg 返回内容
     * @param data    返回数据
     */
    private OssApiResponse(Integer respCode, String respMsg, T data) {
        this.respCode = respCode;
        this.respMsg = respMsg;
        this.data = data;
    }

    /**
     * 构造一个自定义的API返回
     *
     * @param respCode    状态码
     * @param respMsg 返回内容
     * @param data    返回数据
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> of(Integer respCode, String respMsg, T data) {
        return new OssApiResponse<T>(respCode, respMsg, data);
    }

    /**
     * 构造一个成功且不带数据的API返回
     *
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> ofSuccess() {
        return ofSuccess(null);
    }

    /**
     * 构造一个成功且带数据的API返回
     *
     * @param data 返回数据
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> ofSuccess(T data) {
        return ofStatus(Status.OK, data);
    }

    /**
     * 构造一个成功且自定义消息的API返回
     *
     * @param msg 返回内容
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> ofMessage(String msg) {
        return of(Status.OK.getCode(), msg, null);
    }

    /**
     * 构造一个有状态的API返回
     *
     * @param status 状态 {@link Status}
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> ofStatus(ApiStatus status) {
        return ofStatus(status, null);
    }

    /**
     * 构造一个有状态且带数据的API返回
     *
     * @param status 状态 {@link Status}
     * @param data   返回数据
     * @return ApiResponse
     */
    public static <T> OssApiResponse<T> ofStatus(ApiStatus status, T data) {
        return of(status.getCode(), status.getReasonPhrase(), data);
    }
}
