package com.ningdatech.pmapi.org;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.util.SpringUtils;
import com.ningdatech.pmapi.AppTests;
import com.ningdatech.pmapi.gov.entity.BelongOrg;
import com.ningdatech.pmapi.gov.enumeration.BelongTypeEnum;
import com.ningdatech.pmapi.gov.service.IBelongOrgService;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

/**
 * @Classname RegionTest
 * @Description
 * @Date 2023/6/8 14:44
 * @Author PoffyZhang
 */
public class OrgTest extends AppTests {

    @Autowired
    private IDingOrganizationService organizationService;
    @Autowired
    private IBelongOrgService belongOrgService;

    @Test
    public void test() {
        List<DingOrganization> orgs = organizationService.list(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getDivisionCode, "331199"));

        for (DingOrganization org : orgs) {
            List<DingOrganization> inners = organizationService.list(Wrappers.lambdaQuery(DingOrganization.class)
                    .eq(DingOrganization::getParentCode, org.getOrganizationCode()));
            for (DingOrganization inner : inners) {
                mergeRegionCode(inner);
            }
        }
    }

    private void mergeRegionCode(DingOrganization inner) {
        inner.setDivisionCode("331199");
        organizationService.updateById(inner);

        List<DingOrganization> is = organizationService.list(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getParentCode, inner.getOrganizationCode()));
        if (CollUtil.isEmpty(is)) {
            return;
        }
        for (DingOrganization i : is) {
            mergeRegionCode(i);
        }
    }

    @Test
    public void businessStrip() {
        String path = "/Users/wendy/Desktop/省市县条线单位梳理202303016.xlsx";
        ExcelReader reader = ExcelUtil.getReader(path);
        Set<String> orgCodes = new HashSet<>();
        List<BelongOrg> belongOrgs = new ArrayList<>();
        for (int i = 0; i < reader.getSheetCount(); i++) {
            reader.setSheet(i);
            List<Map<String, Object>> maps = reader.readAll();
            for (Map<String, Object> map : maps) {
                String cityName = MapUtil.getStr(map, "市级组织名称");
                if (cityName == null || !cityName.contains("丽水")) {
                    continue;
                }
                String cityCode = MapUtil.getStr(map, "市级组织机构代码");
                if (StrUtil.isBlank(cityCode)) {
                    continue;
                }
                String countyName = MapUtil.getStr(map, "区县组织名称");
                if (StrUtil.isBlank(countyName)) {
                    continue;
                }
                String countyCode = MapUtil.getStr(map, "区县组织机构代码");
                if (StrUtil.isBlank(countyCode)) {
                    continue;
                }
                System.out.printf("市级：%s -> %s  区县：%s -> %s\n", cityName, cityCode, countyName, countyCode);
                BelongOrg county = new BelongOrg();
                county.setBelongCode(cityCode);
                county.setBelongName(cityName);
                county.setOrgCode(countyCode);
                county.setOrgName(countyName);
                county.setOrgType(BelongTypeEnum.ORGANIZATION.name());
                belongOrgs.add(county);
                if (!orgCodes.add(cityCode)) {
                    continue;
                }
                BelongOrg city = new BelongOrg();
                city.setBelongCode(cityCode);
                city.setBelongName(cityName);
                city.setOrgCode(cityCode);
                city.setOrgName(cityName);
                city.setOrgType(BelongTypeEnum.GOV_BUSINESS_STRIP.name());
                belongOrgs.add(city);
            }
            belongOrgs.forEach(System.out::println);
        }
        belongOrgService.saveBatch(belongOrgs);
    }

}
